<?php

class zipfile {

  var $datasec       = array();
  var $ctrl_dir      = array();
  var $eof_ctrl_dir  = "\x50\x4b\x05\x06\x00\x00\x00\x00";
  var $old_offset    = 0;
  var $mimetype = 'application/x-zip'; # or: 'application/octet-stream' to force download!

  function replaceSuffix ($file, $suffix = 'pdf') {
    $arr = explode('.', $file);
    unset($arr[count($arr) - 1]);
    $file = NULL;
    foreach($arr as $v) $file .= $v . '.';
    $file .= $suffix;

    return $file;
  }

  function getDirContent ($dirName = './') {
    if (is_dir($dirName)) {
			$dir = new DeepDir();
			$dir->setDir($dirName);
			$dir->load();
			return $dir->files;
    }
  }

  function addDirContent ($dir = './') {
    foreach ($this->getDirContent($dir) as $input) {
      $this->addFile(str_replace('.//', NULL, $input));
    }
  }

  function addFile ($file, $filename=false) {
    if (is_file($file)){
    	if (!$filename) $filename=$file;
      $this->addFileData(file_get_contents($file), $filename);
   	}
  }

  function unix2DosTime($unixtime = 0) {
    $timearray = ($unixtime == 0) ? getdate() : getdate($unixtime);
    if ($timearray['year'] < 1980) {
      $timearray['year']    = 1980;
      $timearray['mon']     = 1;
      $timearray['mday']    = 1;
      $timearray['hours']   = 0;
      $timearray['minutes'] = 0;
      $timearray['seconds'] = 0;
    }
    return (($timearray['year'] - 1980) << 25) | ($timearray['mon'] << 21) | ($timearray['mday'] << 16) |
      ($timearray['hours'] << 11) | ($timearray['minutes'] << 5) | ($timearray['seconds'] >> 1);
  }

  function addFileData($data, $name, $time = 0) {

    $name     = str_replace('\\', '/', $name);

    $dtime    = dechex($this->unix2DosTime($time));
    $hexdtime = '\x' . $dtime[6] . $dtime[7]
              . '\x' . $dtime[4] . $dtime[5]
              . '\x' . $dtime[2] . $dtime[3]
              . '\x' . $dtime[0] . $dtime[1];

    eval('$hexdtime = "' . $hexdtime . '";');

    $fr   = "\x50\x4b\x03\x04";
    $fr   .= "\x14\x00";            // ver needed to extract
    $fr   .= "\x00\x00";            // gen purpose bit flag
    $fr   .= "\x08\x00";            // compression method
    $fr   .= $hexdtime;             // last mod time and date

    // "local file header" segment
    $unc_len = strlen($data);
    $crc     = crc32($data);
    $zdata   = gzcompress($data);
    $zdata   = substr(substr($zdata, 0, strlen($zdata) - 4), 2); // fix crc bug
    $c_len   = strlen($zdata)+12; # +12: fix by VS!
    $fr      .= pack('V', $crc);             // crc32
    $fr      .= pack('V', $c_len);           // compressed filesize
    $fr      .= pack('V', $unc_len);         // uncompressed filesize
    $fr      .= pack('v', strlen($name));    // length of filename
    $fr      .= pack('v', 0);                // extra field length
    $fr      .= $name;

    // "file data" segment
    $fr .= $zdata;

    // "data descriptor" segment (optional but necessary if archive is not
    // served as file)
    $fr .= pack('V', $crc);                 // crc32
    $fr .= pack('V', $c_len);               // compressed filesize
    $fr .= pack('V', $unc_len);             // uncompressed filesize

    // add this entry to array
    $this -> datasec[] = $fr;
    $new_offset        = strlen(implode('', $this->datasec));

    // now add to central directory record
    $cdrec  = "\x50\x4b\x01\x02";
    $cdrec .= "\x00\x00";                // version made by
    $cdrec .= "\x14\x00";                // version needed to extract
    $cdrec .= "\x00\x00";                // gen purpose bit flag
    $cdrec .= "\x08\x00";                // compression method
    $cdrec .= $hexdtime;                 // last mod time & date
    $cdrec .= pack('V', $crc);           // crc32
    $cdrec .= pack('V', $c_len);         // compressed filesize
    $cdrec .= pack('V', $unc_len);       // uncompressed filesize
    $cdrec .= pack('v', strlen($name) ); // length of filename
    $cdrec .= pack('v', 0 );             // extra field length
    $cdrec .= pack('v', 0 );             // file comment length
    $cdrec .= pack('v', 0 );             // disk number start
    $cdrec .= pack('v', 0 );             // internal file attributes
    $cdrec .= pack('V', 32 );            // external file attributes - 'archive' bit set

    $cdrec .= pack('V', $this -> old_offset ); // relative offset of local header
    $this -> old_offset = $new_offset;

    $cdrec .= $name;

    // optional extra field, file comment goes here
    // save to central directory
    $this -> ctrl_dir[] = $cdrec;

  }

  function file($filename=false) {
    $data = implode(NULL, $this -> datasec);
    $ctrldir = implode(NULL, $this -> ctrl_dir);
    $data .= $ctrldir .
			$this -> eof_ctrl_dir .
			pack('v', sizeof($this -> ctrl_dir)) .  // total # of entries "on this disk"
			pack('v', sizeof($this -> ctrl_dir)) .  // total # of entries overall
			pack('V', strlen($ctrldir)) .           // size of central dir
			pack('V', strlen($data)) .              // offset to start of central dir
			"\x00\x00";                             // .zip file comment length
		if ($filename){
  		header('Content-Type: '.$this->mimetype);
			header('Content-Length: ' . strlen($data));
			header('Expires: '.gmdate('D, d M Y H:i:s').' GMT');
			header('Content-Disposition: attachment; filename="'.$filename.'"');
			header('Pragma: no-cache');
			echo $data;
		}else
			return $data;
	}
}


# CLASS DEEPDIR
class DeepDir{

  var $dir;
  var $files;

  function DeepDir(){
    $this->dir = '';
    $this->files = array();
    $this->dirFILO = new FILO();
  }

  function setDir( $dir ){
    $this->dir = $dir;
    $this->files = array();
    $this->dirFILO->zero();
    $this->dirFILO->push( $this->dir );
  }


  function load(){
    while( $this->curDir = $this->dirFILO->pop() ){
      $this->loadFromCurDir();
    }
  }

  function loadFromCurDir(){
    if ( $handle = @opendir( $this->curDir ) ){
      while ( false !== ( $file = readdir( $handle ) ) ){
        if ( $file == "." || $file == ".." ) continue;
        $filePath = $this->curDir . '/' . $file;
        $fileType = filetype( $filePath );
        if ( $fileType == 'dir' ){
          $this->dirFILO->push( $filePath );
          continue;
        }
        $this->files[] = $filePath;
      }
      closedir( $handle );
    }
    else{
      echo 'error open dir "'.$this->curDir.'"';
    }
  }

} // end class


# CLASS FILO (stack: First In Last Out)
class FILO{

  var $elements;
  
  function FILO(){
    $this->zero();
  }

  function push( $elm ){
    array_push( $this->elements, $elm );
  }

  function pop(){
    return array_pop( $this->elements );
  }

  function zero(){
    $this->elements = array();
  }

} // end class FILO

?>